package net.booden.mobiletictactoe;

import javax.bluetooth.*;
import javax.microedition.io.*;
import java.io.*;
import java.util.Vector;

/**
 *
 * @author Ronald Booden
 */
public class MobileTicTacToeBluetoothClient implements IMobileTicTacToePlayType, Runnable, DiscoveryListener {
    private final char you = 'x';
    private final char opponent = 'o';
    private final int keyOffset = 49;
    private MobileTicTacToeGame game;
    private boolean connected = false;

    public void readKey(int keyCode) {
        if(game != null && (game.getCurrentPlayer() == you || game.isGameEnd()))
        {
            int position = keyCode - keyOffset;
            if(!game.isGameEnd() && position >= 0 && position <=8)
                if(game.move(position) && !game.isGameEnd())
                    //startComputer();
            if(game.isGameEnd() && keyCode == -5)
            {
                game.initGame();
                if(game.getCurrentPlayer() == opponent)
                {
                    //startComputer();
                }
            }
        }
    }

    public String getStatus() {
        if(connected) return "Connected";
        return "Disconnected";
    }

    public String getPlay() {
        if(game != null && game.isGameEnd())
        {
            if(game.getWinner() == you) return "You win";
            if(game.getWinner() == opponent) return "You lose";
            return "No winner this time!";
        }
        if(game != null)
            return game.getCurrentPlayer() == you ? "Your move" : "Wait for opponents move";
        return "Bluetooth play enabled";
    }

    public void setGame(MobileTicTacToeGame game) {
        this.game = game;
    }
    //object used for waiting
    private final Object lock=new Object();
    //vector containing the devices discovered
    private static Vector vecDevices=new Vector();
    private static String connectionURL=null;
    public void startClient() throws IOException {
        //display local device address and name
        LocalDevice localDevice = LocalDevice.getLocalDevice();
        System.out.println("Address: "+localDevice.getBluetoothAddress());
        System.out.println("Name: "+localDevice.getFriendlyName());
        //find devices
        DiscoveryAgent agent = localDevice.getDiscoveryAgent();
        System.out.println("Starting device inquiry...");
        agent.startInquiry(DiscoveryAgent.GIAC, this);
        try {
                synchronized(lock){
                        lock.wait();
                }
        }
        catch (InterruptedException e) {
                e.printStackTrace();
        }
        System.out.println("Device Inquiry Completed. ");
        //print all devices in vecDevices
        int deviceCount=vecDevices.size();
        if(deviceCount <= 0){
                System.out.println("No Devices Found .");
                return;
        }
        else{
                //print bluetooth device addresses and names in the format [ No. address (name) ]
                System.out.println("Bluetooth Devices: ");
                for (int i = 0; i <deviceCount; i++) {
                        RemoteDevice remoteDevice=(RemoteDevice)vecDevices.elementAt(i);
                        System.out.println((i+1)+". "+remoteDevice.getBluetoothAddress()+" ("+remoteDevice.getFriendlyName(true)+")");
                }
        }
        System.out.print("Choose Device index: ");
        InputStreamReader iReader= new InputStreamReader(new InputStream() {

            public int read() throws IOException {
                return 1;
            }
        });
        char[] cbuf = new char[1];
        iReader.read(cbuf);
        //check for spp service
        int index = 1;
        RemoteDevice remoteDevice = (RemoteDevice)vecDevices.elementAt(index-1);
        UUID[] uuidSet = new UUID[1];
        uuidSet[0] = new UUID("4442",false);
        System.out.println("\nSearching for service...");
        agent.searchServices(null,uuidSet,remoteDevice,this);
        try {
                synchronized(lock){
                        lock.wait();
                }
        }
        catch (InterruptedException e) {
                e.printStackTrace();
        }
        if(connectionURL==null){
                System.out.println("Device does not support Simple SPP Service.");
                System.exit(0);
        }
        //connect to the server and send a line of text
        StreamConnection streamConnection=(StreamConnection)Connector.open(connectionURL);
        //send string
        OutputStream outStream=streamConnection.openOutputStream();
        OutputStreamWriter oWriter= new OutputStreamWriter(outStream);
        oWriter.write("1");
        oWriter.flush();
        //read response
        InputStream inStream = streamConnection.openInputStream();
        InputStreamReader iReader2= new InputStreamReader(inStream);
        iReader2.read(cbuf);
        System.out.println(cbuf);
    }
    
    //methods of DiscoveryListener
    public void deviceDiscovered(RemoteDevice btDevice, DeviceClass cod) {
            //add the device to the vector
            if(!vecDevices.contains(btDevice)){
                    vecDevices.addElement(btDevice);
            }
    }
    //implement this method since services are not being discovered
    public void servicesDiscovered(int transID, ServiceRecord[] servRecord) {
            if(servRecord!=null && servRecord.length>0){
                    connectionURL=servRecord[0].getConnectionURL(0,false);
            }
            synchronized(lock){
                    lock.notify();
            }
    }
    //implement this method since services are not being discovered
    public void serviceSearchCompleted(int transID, int respCode) {
            synchronized(lock){
                    lock.notify();
            }
    }
    public void inquiryCompleted(int discType) {
            synchronized(lock){
                    lock.notify();
            }
    }//end method

    public void run() {
    }
}
