package net.booden.mobiletictactoe;

import javax.bluetooth.*;
import javax.microedition.io.*;
import java.io.*;

/**
 *
 * @author Ronald Booden
 */
public class MobileTicTacToeBluetoothServer implements IMobileTicTacToePlayType, Runnable {

    private final char you = 'x';
    private final char opponent = 'o';
    private final int keyOffset = 49;
    private MobileTicTacToeGame game;
    private boolean connected = false;
    
    public void readKey(int keyCode) {
        if(game != null && (game.getCurrentPlayer() == you || game.isGameEnd()))
        {
            int position = keyCode - keyOffset;
            if(!game.isGameEnd() && position >= 0 && position <=8)
                if(game.move(position) && !game.isGameEnd())
                    //startComputer();
            if(game.isGameEnd() && keyCode == -5)
            {
                game.initGame();
                if(game.getCurrentPlayer() == opponent)
                {
                    //startComputer();
                }
            }
        }
    }

    public String getStatus() {
        if(connected) return "Connected";
        return "Disconnected";
    }

    public String getPlay() {
        if(game != null && game.isGameEnd())
        {
            if(game.getWinner() == you) return "You win";
            if(game.getWinner() == opponent) return "You lose";
            return "No winner this time!";
        }
        if(game != null)
            return game.getCurrentPlayer() == you ? "Your move" : "Wait for opponents move";
        return "Bluetooth play enabled";
    }

    public void setGame(MobileTicTacToeGame game) {
        this.game = game;
    }

    public void startServer() throws IOException{
            //Create a UUID for MobileTicTacToe Server
            UUID uuid = new UUID("4442", true);
            //Create the servicve url
            String connectionString = "btspp://localhost:" + uuid +";name=MobileTicTacToe Server";
            //open server url
            StreamConnectionNotifier streamConnNotifier = (StreamConnectionNotifier)Connector.open( connectionString );
            //Wait for client connection
            System.out.println("\nServer Started. Waiting for clients to connect...");
            StreamConnection connection = streamConnNotifier.acceptAndOpen();
            RemoteDevice dev = RemoteDevice.getRemoteDevice(connection);
            System.out.println("Remote device address: "+dev.getBluetoothAddress());
            System.out.println("Remote device name: "+dev.getFriendlyName(true));
            //read string from spp client
            InputStream inStream = connection.openInputStream();
            InputStreamReader iReader = new InputStreamReader(inStream);
            char[] cbuf = new char[1];
            iReader.read(cbuf);
            System.out.println(cbuf);
            //send response to spp client
            OutputStream outStream = connection.openOutputStream();
            OutputStreamWriter oWriter = new OutputStreamWriter(outStream);
            oWriter.write(cbuf);
            oWriter.flush();
            oWriter.close();
            streamConnNotifier.close();
        }

    public void run() {
    }

}
