package net.booden.mobiletictactoe;

/**
 *
 * @author Powerline
 */
public class MobileTicTacToeComputerPlay implements IMobileTicTacToePlayType {

    private final char humanPlayer = 'x';
    private final char computerPlayer = 'o';
    private final int keyOffset = 49;
    private MobileTicTacToeGame game;

    public void readKey(int keyCode) {
        if(game != null && (game.getCurrentPlayer() == humanPlayer || game.isGameEnd()))
        {
            int position = keyCode - keyOffset;
            if(!game.isGameEnd() && position >= 0 && position <=8)
                if(game.move(position) && !game.isGameEnd())
                    startComputer();
            if(game.isGameEnd() && keyCode == -5)
            {
                game.initGame();
                if(game.getCurrentPlayer() == computerPlayer)
                    startComputer();
            }
        }
    }

    public String getStatus() {
        return "Disconnected";
    }

    public String getPlay() {
        if(game != null && game.isGameEnd())
        {
            if(game.getWinner() == humanPlayer) return "You win";
            if(game.getWinner() == computerPlayer) return "You lose";
            return "No winner this time!";
        }
        if(game != null)
            return game.getCurrentPlayer() == humanPlayer ? "Your move" : "Wait for opponents move";
        return "Computer play enabled";
    }

    public void setGame(MobileTicTacToeGame game) {
        this.game = game;
    }

    private void startComputer()
    {
        Thread t = new Thread(new Runnable() {
            public void run() {
                computerMove();
            }
        } );
        t.start();
    }

    private void computerMove() {
        try
        {
            Thread.sleep(1500);
        }
        catch(InterruptedException e) {
            //do nothing
        }
        int move = calculateMove();
        game.move(move);
    }

    private int calculateMove() {
        if (game.getBoardValue(4) == '?') return 4;
        if (game.getBoardValue(0) == '?') return 0;
        //Possible to win
        int win = findWin(computerPlayer);
        if (win != -1) return win;
        //Possible to lose
        win = findWin(humanPlayer);
        if (win != -1) return win;
        for (int i = 0; i < 9; i += 2) //first try the corners
        {
            if (game.getBoardValue(i) == '?')
            {
                return i;
            }
        }
        for (int i = 0; i < 9; i++)
        {
            if (game.getBoardValue(i) == '?')
            {
                return i;
            }
        }
        return 8; /* Just to satisfy the compiler */
    }

    public int findWin(char who)
    {
        int result = checkPossibleWin('?', who, who);
        if (result == -1) result = checkPossibleWin(who, '?', who);
        if (result == -1) result = checkPossibleWin(who, who, '?');
        return result;
    }

    private int checkPossibleWin(char x, char y, char z)
    {
        if ((game.getBoardValue(0) == x) && (game.getBoardValue(1) == y) && (game.getBoardValue(2) == z)) return x == '?' ? 0 : y == '?' ? 1 : 2;
        if ((game.getBoardValue(0) == x) && (game.getBoardValue(3) == y) && (game.getBoardValue(6) == z)) return x == '?' ? 0 : y == '?' ? 3 : 6;
        if ((game.getBoardValue(0) == x) && (game.getBoardValue(4) == y) && (game.getBoardValue(8) == z)) return x == '?' ? 0 : y == '?' ? 4 : 8;
        if ((game.getBoardValue(1) == x) && (game.getBoardValue(4) == y) && (game.getBoardValue(7) == z)) return x == '?' ? 1 : y == '?' ? 4 : 7;
        if ((game.getBoardValue(2) == x) && (game.getBoardValue(4) == y) && (game.getBoardValue(6) == z)) return x == '?' ? 2 : y == '?' ? 4 : 6;
        if ((game.getBoardValue(2) == x) && (game.getBoardValue(5) == y) && (game.getBoardValue(8) == z)) return x == '?' ? 2 : y == '?' ? 5 : 8;
        if ((game.getBoardValue(3) == x) && (game.getBoardValue(4) == y) && (game.getBoardValue(5) == z)) return x == '?' ? 3 : y == '?' ? 4 : 5;
        if ((game.getBoardValue(6) == x) && (game.getBoardValue(7) == y) && (game.getBoardValue(8) == z)) return x == '?' ? 6 : y == '?' ? 7 : 8;
        return -1;
    }
}
