package net.booden.mobiletictactoe;

import java.io.IOException;
import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;

/**
 *
 * @author Ronald Booden
 */
public class MobileTicTacToeMIDlet extends MIDlet implements CommandListener {

    private Command exitCommand; // The exit command
    private Command menuCommand; // The connect command
    private Command selectCommand; // The exit command
    private List menu;

    private MobileTicTacToeGame game;

    public MobileTicTacToeMIDlet() {
        //Init commands
        menuCommand = new Command("Menu", Command.SCREEN, 0);
        exitCommand = new Command("Exit", Command.EXIT, 0);
        selectCommand = new Command("Ok", Command.SCREEN, 0);
        //Init menu
        if(supportsBluetooth())
        {
            String[] menuItems = {"Local Play", "Play Computer", "Wait for Connection", "Connect to Device"};
            menu = new List("MobileTicTacToe", List.IMPLICIT, menuItems, null);
        }
        else
        {
            String[] menuItems = {"Local Play", "Play Computer"};
            menu = new List("MobileTicTacToe", List.IMPLICIT, menuItems, null);
        }
        menu.addCommand(exitCommand);
        menu.addCommand(selectCommand);
        menu.setCommandListener(this);
    }

    private static boolean supportsBluetooth() {
        try
        {
            Class.forName("javax.bluetooth.LocalDevice");
            return true;
        }
        catch(Throwable t) {
            return false;
        }
    }

    public void startApp() {
        ShowMenu();
    }

    public void pauseApp() {
    }

    public void destroyApp(boolean unconditional) {
    }

    public void commandAction(Command c, Displayable s) {
        if (c == exitCommand) {
            destroyApp(true);
            notifyDestroyed();
            return;
        }
        if (c == menuCommand)
        {
            ShowMenu();
            return;
        }
        switch (menu.getSelectedIndex ()) {
        case 0:
            CreateCanvas(new MobileTicTacToeLocalPlay());
            break;
        case 1:
            CreateCanvas(new MobileTicTacToeComputerPlay());
            break;
        case 2:
            MobileTicTacToeBluetoothServer btServer = new MobileTicTacToeBluetoothServer();
            try
            {
                btServer.startServer();
                CreateCanvas(btServer);
            }
            catch(IOException e) {}
            break;
        case 3:
            MobileTicTacToeBluetoothClient btClient = new MobileTicTacToeBluetoothClient();
            try
            {
                btClient.startClient();
                CreateCanvas(btClient);
            }
            catch(IOException e) {}
            break;
        }
    }

    private void CreateCanvas(IMobileTicTacToePlayType playType)
    {
        MobileTicTacToeCanvas mc = new MobileTicTacToeCanvas();
        mc.addCommand(exitCommand);
        mc.addCommand(menuCommand);
        mc.setCommandListener(this);
        Display.getDisplay(this).setCurrent(mc);
        game = new MobileTicTacToeGame(mc, playType);
    }

    private void ShowMenu()
    {
        Display.getDisplay(this).setCurrent(menu);
    }

}
