package net.booden.mobiletictactoe;

import javax.bluetooth.*;
import java.io.*;
import javax.microedition.io.*;
import java.util.*;

/**
 *
 * @author Ronald Booden
 */
public class BluetoothClient implements IClient, Runnable, DiscoveryListener {

    //Implemented custom connection polling due to the behaviour I experienced
    //with my mobile device and sudden disconnection(I was testing for this)
    //on bluetooth. Connections should be closed correctly for the connection
    //to be available again for reconnection.

    //object used for waiting
    private final Object lock = new Object();
    //vector containing the devices discovered
    private Vector vecDevices = new Vector();
    private String connectionURL;

    private DiscoveryAgent agent;
    private int search;
    private StreamConnection streamConnection;
    private InputStreamReader iReader;
    private OutputStreamWriter oWriter;
    private Thread startThread;
    private String uuid;
    private boolean connected = false;
    private IReciever reciever;
    private int numChars;
    private int deviceIndex;
    private InputStream inStream;
    private OutputStream outStream;
    private char[] exit;
    private char[] polling;
    private Date pollingTime;

    public BluetoothClient(String uuid)
    {
        this.uuid = uuid;
    }

    public void setDeviceIndex(int index)
    {
        deviceIndex = index;
        startThread = new Thread(this);
        startThread.start();
    }

    public boolean isConnected() {
        return connected;
    }

    public void setReciever(IReciever reciever, int numChars) {
        exit = new char[numChars];
        for(int i = 0; i < numChars; i++) exit[i] = 'x';
        polling = new char[numChars];
        for(int i = 0; i < numChars; i++) polling[i] = '?';
        this.reciever = reciever;
        this.numChars = numChars;
    }

    public void send(char[] value) {
        try
        {
            oWriter.write(value);
            oWriter.flush();
        }
        catch(IOException e) 
        {
            setDisconnected();
        }
    }

    public void disconnect() {
        if(connected)
        {
            connected = false;
            if(agent != null)
            {
                agent.cancelInquiry(this);
                agent.cancelServiceSearch(search);
                agent = null;
            }
            if(streamConnection != null)
            {
                try
                {
                    send(exit);
                    streamConnection.close();
                }
                catch(IOException e) { }
                streamConnection = null;
            }
            if(inStream != null)
            {
                try
                {
                    inStream.close();
                }
                catch(IOException e) { }
                inStream = null;
            }
            if(outStream != null)
            {
                try
                {
                    outStream.close();
                }
                catch(IOException e) { }
                outStream = null;
            }
            if(iReader != null)
            {
                try
                {
                    iReader.close();
                }
                catch(IOException e) { }
                iReader = null;
            }
            if(oWriter != null)
            {
                try
                {
                    oWriter.close();
                }
                catch(IOException e) { }
                oWriter = null;
            }
            if(startThread != null && startThread.isAlive())
            {
                try
                {
                    startThread.join();
                }
                catch(InterruptedException e) { }
            }
        }
        System.gc();
    }

    public void run() {
        try
        {
            if(!connectToDevice(deviceIndex)) return;
            //Setup IO
            outStream = streamConnection.openOutputStream();
            oWriter = new OutputStreamWriter(outStream);
            inStream = streamConnection.openInputStream();
            iReader= new InputStreamReader(inStream);
            connected = true;
            setConnected();
            pollingTime = new Date();
            readMoves();
        }
        catch(IOException e)
        {
            setDisconnected();
        }
    }

    private void setConnected()
    {
        Thread t = new Thread(new Runnable()
        {
            public void run() {
                reciever.notifyConnected();
            }
        });
        t.start();
    }

    private void setDisconnected()
    {
        Thread t = new Thread(new Runnable()
        {
            public void run() {
                disconnect();
                reciever.notifyDisconnected();
            }
        });
        t.start();
    }

    public String[] discoverDevices()
    {
        String[] devices = new String[1];
        devices[0] = "No devices found";
        try
        {
            //find devices
            LocalDevice localDevice = LocalDevice.getLocalDevice();
            agent = localDevice.getDiscoveryAgent();
            agent.startInquiry(DiscoveryAgent.GIAC, this);
            try
            {
                synchronized (lock) {
                    lock.wait();
                }
            }
            catch (InterruptedException e) { }

            //get all devices in vecDevices
            int deviceCount = vecDevices.size();
            if (deviceCount <= 0) {
                return devices;
            }
            else
            {
                //print bluetooth device addresses and names in the format [ No. address (name) ]
                devices = new String[deviceCount];
                for (int i = 0; i < deviceCount; i++)
                {
                    RemoteDevice remoteDevice = (RemoteDevice) vecDevices.elementAt(i);
                    devices[i] = remoteDevice.getFriendlyName(true);
                }
            }
        }
        catch(IOException e) { }
        return devices;
    }

    private boolean connectToDevice(int index)
    {
        //Create a UUID for Service
        UUID SERVICE = new UUID(uuid, false);

        RemoteDevice remoteDevice = (RemoteDevice) vecDevices.elementAt(index);
        UUID[] uuidSet = new UUID[1];
        uuidSet[0] = SERVICE;

        try
        {
            search = agent.searchServices(null, uuidSet, remoteDevice, this);
        }
        catch(Exception e) { }
        try {
            synchronized (lock) {
                lock.wait();
            }
        } 
        catch (InterruptedException e)
        {
            return false;
        }
        if (connectionURL == null) {
            return false;
        }
        //connect to the server and send a line of text
        try {
            streamConnection = (StreamConnection)Connector.open(connectionURL);
        }
        catch (IOException e)
        {
            return false;
        }
        finally
        {
            agent = null;
        }
        return true;
    }

    private void readMoves() {
        char[] cbuf = new char[numChars];
        while(connected)
        {
            try
            {
                if(iReader.ready())
                {
                    iReader.read(cbuf);
                    if(arraysEquals(cbuf, exit))
                    {
                        setDisconnected();
                        return;
                    }
                    if(reciever != null && !arraysEquals(cbuf, polling))
                        reciever.recieve(cbuf);

                    pollingTime = new Date();
                }
                else
                {
                    send(polling);
                    if(pollingTime != null && pollingTime.getTime() < (new Date()).getTime() - 1500)
                    {
                        setDisconnected();
                        return;
                    }
                    Thread.sleep(100);
                    Thread.yield();
                }
            }
            catch(IOException e)
            {
                setDisconnected();
            }
            catch(InterruptedException e) { }
        }
    }
    
    private boolean arraysEquals(char[] one, char[] two)
    {
        if(one.length != two.length) return false;
        for(int i = 0; i < one.length; i++)
            if(one[i] != two[i]) return false;
        return true;
    }

    //methods of DiscoveryListener
    public void deviceDiscovered(RemoteDevice btDevice, DeviceClass cod) {
            //add the device to the vector
            if(!vecDevices.contains(btDevice)){
                    vecDevices.addElement(btDevice);
            }
    }
    //implement this method since services are not being discovered
    public void servicesDiscovered(int transID, ServiceRecord[] servRecord) {
            if(servRecord!=null && servRecord.length>0){
                    connectionURL=servRecord[0].getConnectionURL(0,false);
            }
            synchronized(lock){
                    lock.notify();
            }
    }
    //implement this method since services are not being discovered
    public void serviceSearchCompleted(int transID, int respCode) {
            synchronized(lock){
                    lock.notify();
            }
    }
    public void inquiryCompleted(int discType) {
            synchronized(lock){
                    lock.notify();
            }
    }//end method

}
