package net.booden.mobiletictactoe;

import java.io.*;
import javax.microedition.lcdui.*;

/**
 *
 * @author Ronald Booden
 */
public class MobileTicTacToeCanvas extends Canvas implements IMobileTicTacToeGUI {

    private class Point
    {
        private int _x;
        private int _y;

        Point(int x, int y)
        {
            _x = x;
            _y = y;
        }

        public int getX() { return _x; }
        public int getY() { return _y; }
    }

    private Image imgIcon;
    private Image imgX;
    private Image imgO;

    private char board[] = new char[9];
    private Point pos[] = new Point[9];
    private String status = "";
    private String play = "";
    private IMobileTicTacToeInputReader inputReader;
    private int leftMargin;
    private int topMargin;

    private static final int TL_ANCHOR = Graphics.TOP | Graphics.LEFT;

    public MobileTicTacToeCanvas()
    {
        try
        {
            imgIcon = Image.createImage("/nttticon.png");
            imgX = Image.createImage("/x.png");
            imgO = Image.createImage("/o.png");
        }
        catch(IOException e)
        {
            throw new RuntimeException("Unable to load Image: " + e);
        }
        initPoints();
    }

    protected void keyReleased(int keyCode) {
        if(inputReader != null)
            inputReader.readKey(keyCode);
    }

    public void pointerReleased (int x, int y)
    {
        int posX = x - leftMargin;
        int posY = y - topMargin;
        for(int i = 0; i < pos.length; i++)
        {
            if(posX > pos[i].getX() && posX < pos[i].getX() + imgX.getWidth())
            {
                if(posY > pos[i].getY() && posY < pos[i].getY() + imgX.getHeight())
                {
                    inputReader.readPointer(i);
                }
            }
        }
    }

    private void initPoints()
    {
        int i;
        int x = 1, y = 24;
        for(i = 0; i < 9; i++) {
            pos[i] = new Point(x,y);
            x = x + 48;
            if (i == 2) {
                x = 1;
                y = y + 47;
            }
            if (i == 5) {
                x = 1;
                y = y + 47;
            }
        }
    }
    
    protected void paint(Graphics g) {
        //calculate margins for centering the tictactoe area
        leftMargin = (g.getClipWidth() - 144) / 2;
        topMargin = (g.getClipHeight() - 195) / 2;

        //background
        g.setColor(128, 128, 128);
        g.fillRect(0, 0, g.getClipWidth(), g.getClipHeight());

        //Koptekst
        g.setColor(0x7A97E0);
        g.fillRect(leftMargin, topMargin, 144, 20);
        g.drawImage(imgIcon, leftMargin + 2, topMargin + 2, TL_ANCHOR);
        Font fontMedium = Font.getFont(Font.FACE_SYSTEM, Font.STYLE_PLAIN, Font.SIZE_MEDIUM);
        g.setFont(fontMedium);
        g.setColor(255, 255, 255);
        g.drawString("MobileTicTacToe", leftMargin + 20, topMargin + 3, TL_ANCHOR);

        //spelvlak

        g.fillRect(leftMargin + 0, topMargin + 20, 144, 142);
        g.setColor(0,0,0);
        g.drawRect(leftMargin + 0, topMargin + 20, 144, 142);
        //lines
        g.drawLine(leftMargin + 0, topMargin + 66, leftMargin + 144, topMargin + 66);
        g.drawLine(leftMargin + 0, topMargin + 116, leftMargin + 144, topMargin + 116);
        g.drawLine(leftMargin + 47, topMargin + 20, leftMargin + 47, topMargin + 162);
        g.drawLine(leftMargin + 96, topMargin + 20, leftMargin + 96, topMargin + 162);

        //X of O plaatjes
        for(int i = 0; i < 9; i++)
        {
            if(board[i] == 'x')
            {
                g.drawImage(imgX,leftMargin + (int)(pos[i].getX()),topMargin + (int)(pos[i].getY()), TL_ANCHOR);
            }
            else
            {
                if(board[i] == 'o')
                {
                      g.drawImage(imgO,leftMargin + (int)(pos[i].getX()),topMargin + (int)(pos[i].getY()),TL_ANCHOR);
                }
            }
        }

        //status
        g.setColor(255, 255, 255);
        g.drawString(status, leftMargin + 1, topMargin + 165, TL_ANCHOR);
        g.drawString(play, leftMargin + 1, topMargin + 179, TL_ANCHOR);
    }

    public void update(char[] board) {
        this.board = board;
        this.repaint();
    }

    public void setStatus(String status) {
        this.status = status;
    }

    public void setPlay(String play) {
        this.play = play;
    }

    public void setKeyReader(IMobileTicTacToeInputReader reader) {
        this.inputReader = reader;
    }
}
