package net.booden.mobiletictactoe;

import javax.bluetooth.*;
import java.io.*;
import java.util.*;
import javax.microedition.io.*;

/**
 *
 * @author Ronald Booden
 */
public class BluetoothServer implements IService, Runnable {

    //Implemented custom connection polling due to the behaviour I experienced
    //with my mobile device and sudden disconnection(I was testing for this)
    //on bluetooth. Connections should be closed correctly for the connection
    //to be available again for reconnection.
    
    private StreamConnectionNotifier streamConnNotifier;
    private StreamConnection connection;
    private InputStreamReader iReader;
    private OutputStreamWriter oWriter;
    private Thread startThread;
    private String uuid;
    private String serviceName;
    private boolean connected = false;
    private IReciever reciever;
    private int numChars;
    private InputStream inStream;
    private OutputStream outStream;
    private char[] exit;
    private char[] polling;
    private Date pollingTime;

    public BluetoothServer(String uuid, String serviceName)
    {
        this.uuid = uuid;
        this.serviceName = serviceName;
        startThread = new Thread(this);
        startThread.start();
    }

    public boolean isConnected() {
        return connected;
    }

    public void setReciever(IReciever reciever, int numChars) {
        exit = new char[numChars];
        for(int i = 0; i < numChars; i++) exit[i] = 'x';
        polling = new char[numChars];
        for(int i = 0; i < numChars; i++) polling[i] = '?';
        this.reciever = reciever;
        this.numChars = numChars;
    }

    public void send(char[] value) {
        try
        {
            oWriter.write(value);
            oWriter.flush();
        }
        catch(IOException e) 
        {
            setDisconnected();
        }
    }

    public void disconnect() {
        if(connected)
        {
            connected = false;
            if(streamConnNotifier != null)
            {
                try
                {
                    streamConnNotifier.close();
                }
                catch(IOException e) { }
                streamConnNotifier = null;
            }
            if(connection != null)
            {
                try
                {
                    send(exit);
                    connection.close();
                }
                catch(IOException e) { }
                connection = null;
            }
            if(inStream != null)
            {
                try
                {
                    inStream.close();
                }
                catch(IOException e) { }
                inStream = null;
            }
            if(outStream != null)
            {
                try
                {
                    outStream.close();
                }
                catch(IOException e) { }
                outStream = null;
            }
            if(iReader != null)
            {
                try
                {
                    iReader.close();
                }
                catch(IOException e) { }
                iReader = null;
            }
            if(oWriter != null)
            {
                try
                {
                    oWriter.close();
                }
                catch(IOException e) { }
                oWriter = null;
            }
            if(startThread != null && startThread.isAlive())
            {
                try
                {
                    startThread.join();
                }
                catch(InterruptedException e) { }
            }
        }
        System.gc();
    }

    public void run() {
        try
        {
            waitForConnection();

            //Setup IO
            inStream = connection.openInputStream();
            iReader = new InputStreamReader(inStream);
            outStream = connection.openOutputStream();
            oWriter = new OutputStreamWriter(outStream);
            connected = true;
            setConnected();
            pollingTime = new Date();
            readMoves();
        }
        catch(IOException e)
        {
            setDisconnected();
        }
    }

    private void setConnected()
    {
        Thread t = new Thread(new Runnable()
        {
            public void run() {
                reciever.notifyConnected();
            }
        });
        t.start();
    }

    private void setDisconnected()
    {
        Thread t = new Thread(new Runnable()
        {
            public void run() {
                disconnect();
                reciever.notifyDisconnected();
            }
        });
        t.start();
    }

    private void waitForConnection() throws IOException {
        //Create a UUID for Service
        UUID SERVICE = new UUID(uuid, false);
        //Create the servicve url
        String connectionString = "btspp://localhost:" + SERVICE + ";name=" + serviceName;

        //open server url
        streamConnNotifier = (StreamConnectionNotifier) Connector.open( connectionString );
        //Wait for client connection
        try
        {
            connection = streamConnNotifier.acceptAndOpen();
        }
        finally
        {
            streamConnNotifier.close();
            streamConnNotifier = null;
        }
    }

    private void readMoves() {
        char[] cbuf = new char[numChars];
        while(connected)
        {
            try
            {
                if(iReader.ready())
                {
                    iReader.read(cbuf);
                    if(arraysEquals(cbuf, exit))
                    {
                        setDisconnected();
                        return;
                    }
                    if(reciever != null && !arraysEquals(cbuf, polling))
                        reciever.recieve(cbuf);

                    pollingTime = new Date();
                }
                else
                {
                    send(polling);
                    if(pollingTime != null && pollingTime.getTime() < (new Date()).getTime() - 1500)
                    {
                        setDisconnected();
                        return;
                    }
                    Thread.sleep(100);
                    Thread.yield();
                }
            }
            catch(IOException e)
            {
                setDisconnected();
            }
            catch(InterruptedException e) { }
        }
    }

    private boolean arraysEquals(char[] one, char[] two)
    {
        if(one.length != two.length) return false;
        for(int i = 0; i < one.length; i++)
            if(one[i] != two[i]) return false;
        return true;
    }

}
