package net.booden.mobiletictactoe;

/**
 *
 * @author Ronald Booden
 */
public class MobileTicTacToeClient implements IMobileTicTacToePlayType, IReciever {

    private final char you = 'o';
    private final char opponent = 'x';
    private final int keyOffset = 49;
    private MobileTicTacToeGame game;
    private IClient client;

    public MobileTicTacToeClient(IClient clnt) {
        if(clnt != null)
        {
            this.client = clnt;
            this.client.setReciever(this, 1);
        }
    }

    public void dispose() {
        if(client != null)
        {
            if(client.isConnected())
            {
                client.disconnect();
                client = null;
            }
        }
    }

    public void readKey(int keyCode) {
        int position = keyCode - keyOffset;
        processMove(position);
    }

    public void readPointer(int areaNumber) {
        if(game.isGameEnd()) processMove(-54);
        else processMove(areaNumber);
    }

    private void processMove(int pos) {
        if(game != null && (game.getCurrentPlayer() == you || game.isGameEnd() || client == null || !client.isConnected()))
        {
            char move = Integer.toString(pos).charAt(0);
            if(!game.isGameEnd() && pos >= 0 && pos <=8)
                if(game.move(pos))
                {
                    if(client != null && client.isConnected())
                    {
                        sendMove(move);
                    }
                }
            if(game.isGameEnd() && pos == -54)
            {
                if(client != null && client.isConnected()) sendMove('9');
                game.initGame();
            }
        }
    }

    public void recieve(char[] value) {
        try
        {
            String val = String.valueOf(value);
            int position = Integer.parseInt(val);
            if(game != null && (game.getCurrentPlayer() == opponent || game.isGameEnd()))
            {
                if(!game.isGameEnd() && position >= 0 && position <=8)
                    game.move(position);
                if(game.isGameEnd() && position == 9)
                {
                    game.initGame();
                }
            }
        }
        catch(NumberFormatException e) { }
    }

    private void sendMove(char move)
    {
        client.send(new char[] { move });
    }

    public String getStatus() {
        if(client != null && client.isConnected()) return "Connected";
        return "Disconnected";
    }

    public String getPlay() {
        if(game != null && game.isGameEnd())
        {
            if(client != null && client.isConnected())
            {
                if(game.getWinner() == you) return "You win";
                if(game.getWinner() == opponent) return "You lose";
                return "No winner this time!";
            }
            else
            {
                if(game.getWinner() == 'x') return "X wins";
                if(game.getWinner() == 'o') return "O wins";
                return "No winner this time!";
            }
        }
        if(game != null)
        {
            if(client != null && client.isConnected())
                return game.getCurrentPlayer() == you ? "Your move" : "Wait for opponents move";
            else
                return "Local play enabled";
        }
        return "Client play enabled";
    }

    public void setGame(MobileTicTacToeGame game) {
        this.game = game;
    }

    public void notifyConnected() {
        game.startFirstGame();
    }

    public void notifyDisconnected() {
        client = null;
        this.game.initGame();
    }

}
