package net.booden.mobiletictactoe;

/**
 *
 * @author Ronald Booden
 */
public class MobileTicTacToeGame {

    private IMobileTicTacToeGUI gui;
    private IMobileTicTacToePlayType playType;
    private boolean gameEnd = false;
    private char[] board = new char[9];
    private char startPlayer = 'o';
    private char currentPlayer = 'x';
    private char winner = '?';

    public void dispose() {
        if(playType != null)
            playType.dispose();
    }

    public MobileTicTacToeGame(IMobileTicTacToeGUI gui, IMobileTicTacToePlayType playType)
    {
        this.gui = gui;
        this.playType = playType;
        this.playType.setGame(this);
        this.gui.setStatus(playType.getStatus());
        this.gui.setKeyReader(playType);
        initGame();
    }

    public boolean isGameEnd(){
        return gameEnd;
    }
    
    public void initGame()
    {
        for(int i = 0; i < 9; i++)
        {
          board[i] = '?';
        }
        startPlayer = (startPlayer == 'x') ? 'o' : 'x';
        currentPlayer = startPlayer;
        winner = '?';
        gameEnd = false;
        gui.setStatus(playType.getStatus());
        gui.setPlay(playType.getPlay());
        gui.update(board);
    }

    public void startFirstGame() {
        for(int i = 0; i < 9; i++)
        {
          board[i] = '?';
        }
        startPlayer = 'x';
        currentPlayer = startPlayer;
        winner = '?';
        gameEnd = false;
        gui.setStatus(playType.getStatus());
        gui.setPlay(playType.getPlay());
        gui.update(board);
    }

    public char getWinner() {
        return winner;
    }

    public char getCurrentPlayer() {
        return currentPlayer;
    }

    public char getBoardValue(int position) {
        if(position >= 0 && position < 9)
        {
            return board[position];
        }
        return ' ';
    }

    public boolean move(int position)
    {
        if(position >= 0 && position < 9)
        {
            if(board[position] == '?')
            {
                board[position] = currentPlayer;
                if(checkWin(currentPlayer))
                {
                    winner = currentPlayer;
                    gameEnd = true;
                }
                else
                {
                    boolean check = false;
                    for(int j = 0; (j < 9) && !check; j++)
                    {
                        if(board[j] == '?') check = true;
                    }
                    if (!check)
                    {
                        gameEnd = true;
                    }
                }
                currentPlayer = (currentPlayer == 'x') ? 'o' : 'x';
                gui.setStatus(playType.getStatus());
                gui.setPlay(playType.getPlay());
                gui.update(board);
                return true;
            }
        }
        return false;
    }

    private boolean checkWin(char who)
    {
        if ((board[0] == who) && (board[1] == who) && (board[2] == who)) return true;
        if ((board[0] == who) && (board[3] == who) && (board[6] == who)) return true;
        if ((board[0] == who) && (board[4] == who) && (board[8] == who)) return true;
        if ((board[1] == who) && (board[4] == who) && (board[7] == who)) return true;
        if ((board[2] == who) && (board[4] == who) && (board[6] == who)) return true;
        if ((board[2] == who) && (board[5] == who) && (board[8] == who)) return true;
        if ((board[3] == who) && (board[4] == who) && (board[5] == who)) return true;
        if ((board[6] == who) && (board[7] == who) && (board[8] == who)) return true;
        return false;
    }

}
