package net.booden.mobiletictactoe;

import javax.bluetooth.LocalDevice;
import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;

/**
 *
 * @author Ronald Booden
 */
public class MobileTicTacToeMIDlet extends MIDlet implements CommandListener {

    private Command exitCommand; // The exit command
    private Command menuCommand; // The connect command
    private Command selectCommand; // The exit command
    private List menu;
    private List devices;
    private IClient client;
    private boolean selectDevice = true;
    private String mtttserviceuuid = "26ED8DC056B711DE8A390800200C9A66";

    private MobileTicTacToeGame game;

    public MobileTicTacToeMIDlet() {
        //Init commands
        menuCommand = new Command("Menu", Command.BACK, 0);
        exitCommand = new Command("Exit", Command.EXIT, 0);
        selectCommand = new Command("Ok", Command.BACK, 0);
        //Init menu
        if(supportsBluetooth())
        {
            String[] menuItems = {"Local Play", "Play Computer", "Wait for BT Connection", "Connect to BT Device"};
            menu = new List("MobileTicTacToe", List.IMPLICIT, menuItems, null);
        }
        else
        {
            String[] menuItems = {"Local Play", "Play Computer"};
            menu = new List("MobileTicTacToe", List.IMPLICIT, menuItems, null);
        }
        menu.addCommand(exitCommand);
        menu.addCommand(selectCommand);
        menu.setCommandListener(this);
    }

    private static boolean supportsBluetooth() {
        try
        {
            Class.forName("javax.bluetooth.LocalDevice");
            return true;
        }
        catch(Throwable t) {
            return false;
        }
    }

    public void startApp() {
        ShowMenu();
    }

    public void pauseApp() {
    }

    public void destroyApp(boolean unconditional) {
        if(game != null)
        {
            game.dispose();
            game = null;
            System.gc();
        }
    }

    public void commandAction(Command c, Displayable s) {
        if (c == exitCommand) {
            destroyApp(true);
            notifyDestroyed();
            return;
        }
        if (c == menuCommand)
        {
            selectDevice = true;
            if(game != null)
            {
                game.dispose();
                game = null;
            }
            System.gc();
            ShowMenu();
            return;
        }
        switch (menu.getSelectedIndex ()) {
        case 0:
            CreateCanvas(new MobileTicTacToeLocalPlay());
            break;
        case 1:
            CreateCanvas(new MobileTicTacToeComputerPlay());
            break;
        case 2:
            IService service = new BluetoothServer(mtttserviceuuid, "MobileTicTacToe Service");
            CreateCanvas(new MobileTicTacToeServer(service));
            break;
        case 3:
            if(selectDevice)
            {
                selectDevice = false;
                BluetoothClient bt = new BluetoothClient(mtttserviceuuid);
                String[] dev = bt.discoverDevices();
                client = bt;
                devices = new List("Devices", List.IMPLICIT, dev, null);
                devices.addCommand(menuCommand);
                devices.addCommand(selectCommand);
                devices.setCommandListener(this);
                Display.getDisplay(this).setCurrent(devices);
            }
            else
            {
                selectDevice = true;
                BluetoothClient bt = (BluetoothClient)client;
                bt.setDeviceIndex(devices.getSelectedIndex());
                CreateCanvas(new MobileTicTacToeClient(client));
            }
            break;
        }
    }

    private class CheckSize extends Canvas
    {
        protected void paint(Graphics g) {
        }
    }

    private void CreateCanvas(IMobileTicTacToePlayType playType)
    {
        Canvas mc;
        Canvas c = new CheckSize();
        int height = c.getHeight();
        int width = c.getWidth();
        if(width > 144 && height > 194)
            mc = new MobileTicTacToeCanvas();
        else if(width > 108 && height > 147)
            mc = new MobileTicTacToeSmallCanvas();
        else
            mc = new MobileTicTacToeTinyCanvas();

        mc.addCommand(exitCommand);
        mc.addCommand(menuCommand);
        mc.setCommandListener(this);
        Display.getDisplay(this).setCurrent(mc);
        game = new MobileTicTacToeGame((IMobileTicTacToeGUI)mc, playType);
    }

    private void ShowMenu()
    {
        Display.getDisplay(this).setCurrent(menu);
    }

}
